<?php
/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2 as     *
 *   published by the Free Software Foundation.                            *
 *                                                                         *
 *  Caching Kolab driver developed by KONSEC GmbH, sponsored by WU Wien    *
 *  copyright            : (c) 2006-2007 by Thomas Jarosch                 *
 *  email                : opensource@intra2net.com                        *
 ***************************************************************************/

// C-Client based IMAP implementation
class Kolab_IMAP
{
    // private vars
    var $imap;
    var $base_mbox;
    var $mbox;

    function connect($server, $login, $password, $port = 143, $tls=false)
    {
        $options = "";
        if (!$tls)
            $options = "/notls";

        $this->base_mbox = "{".$server.":".$port.$options."}";
        $this->mbox = $this->base_mbox;

        // IMAP connection reuse
        $do_connect = true;
        if (isset($GLOBALS["_KOLAB_IMAP_SERVER"])
            && isset($GLOBALS["_KOLAB_IMAP_LOGIN"])
            && $server == $GLOBALS["_KOLAB_IMAP_SERVER"]
            && $login == $GLOBALS["_KOLAB_IMAP_LOGIN"])
        {
            $this->imap = $GLOBALS["_KOLAB_IMAP_RESOURCE"];
            $do_connect = false;
        }
        if ($do_connect)
            $this->imap = imap_open($this->base_mbox, $login, $password, OP_HALFOPEN);

        $GLOBALS["_KOLAB_IMAP_SERVER"] = $server;
        $GLOBALS["_KOLAB_IMAP_LOGIN"] = $login;
        $GLOBALS["_KOLAB_IMAP_RESOURCE"] = $this->imap;
    }

    function select($folder)
    {
        $this->mbox = $this->base_mbox.$folder;
        return imap_reopen($this->imap, $this->mbox);
    }

    // Return IMAP status of current folder
    function folder_status()
    {
        $status = imap_status_current($this->imap, SA_MESSAGES | SA_UIDVALIDITY | SA_UIDNEXT);

        $result = array();
        $result["uidvalidity"] = $status->uidvalidity;
        $result["uidnext"] = $status->uidnext;

        return $result;
    }

    // Find messages
    function get_uids()
    {
        $uids = imap_search($this->imap, "UNDELETED", SE_UID);

        if (!is_array($uids))
            $uids = array();

        return $uids;
    }

    // Get header from mailbox
    function get_message_header($uid, $peek_for_body=true)
    {
        $flags = FT_UID;
        if ($peek_for_body)
            $flags |= FT_PREFETCHTEXT;

        return imap_fetchheader($this->imap, $uid, $flags);
    }

    // Get message from mailbox
    function get_message_body($uid)
    {
        return imap_body($this->imap, $uid, FT_UID);
    }

    function get_message($uid)
    {
        $header = $this->get_message_header($uid);
        if ($header === false)
            return false;

        $body = $this->get_message_body($uid);
        if ($body === false)
            return false;

        return $header.$body;
    }

    function append_message($msg)
    {
        return imap_append($this->imap, $this->mbox, $msg);
    }

    function move_message($uid, $new_folder)
    {
        return imap_mail_move($this->imap, $uid, $new_folder, CP_UID);
    }

    function delete_message($uid)
    {
        return imap_delete($this->imap, $uid, FT_UID);
    }

    function undelete_message($uid)
    {
        return imap_undelete($this->imap, $uid, FT_UID);
    }

    function expunge()
    {
        return imap_expunge($this->imap);
    }
}
