<?php
/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2 as     *
 *   published by the Free Software Foundation.                            *
 *                                                                         *
 *  Caching Kolab driver developed by KONSEC GmbH, sponsored by WU Wien    *
 *  copyright            : (c) 2006-2007 by Thomas Jarosch                 *
 *  email                : opensource@intra2net.com                        *
 ***************************************************************************/

require_once "Net/IMAP.php";

// PEAR based IMAP implementation
// TODO: Error handling
class Kolab_IMAP
{
    // private vars
    var $imap;
    var $uid_translation;

    function connect($server, $login, $password, $port = 143, $tls=false)
    {
        // IMAP connection reuse
        $do_connect = true;
        if (isset($GLOBALS["_KOLAB_IMAP_SERVER"])
            && isset($GLOBALS["_KOLAB_IMAP_LOGIN"])
            && $server == $GLOBALS["_KOLAB_IMAP_SERVER"]
            && $login == $GLOBALS["_KOLAB_IMAP_LOGIN"])
        {
            $this->imap = $GLOBALS["_KOLAB_IMAP_RESOURCE"];
            $do_connect = false;
        }
        if ($do_connect) {
            $this->imap = &new Net_IMAP($server, $port);
            $this->imap->login($login, $password, true, false);
        }


        $GLOBALS["_KOLAB_IMAP_SERVER"] = $server;
        $GLOBALS["_KOLAB_IMAP_LOGIN"] = $login;
        $GLOBALS["_KOLAB_IMAP_RESOURCE"] = $this->imap;
    }

    function select($folder)
    {
        return $this->imap->selectMailbox($folder);
    }

    // Return IMAP status of current folder
    function folder_status()
    {
        $result = array();

        $mailbox = $this->imap->getCurrentMailbox();

        // Net_IMAP is not very efficent here
        $ret = $this->imap->cmdStatus( $mailbox , "UIDVALIDITY" );
        $result["uidvalidity"] = $ret["PARSED"]["STATUS"]["ATTRIBUTES"]["UIDVALIDITY"];

        $ret = $this->imap->cmdStatus( $mailbox , "UIDNEXT" );
        $result["uidnext"] = $ret["PARSED"]["STATUS"]["ATTRIBUTES"]["UIDNEXT"];

        return $result;
    }

    // Find messages
    function get_uids()
    {
        $uids = $this->imap->search("UNDELETED", true);
        $this->uid_translation = array();

        if (!is_array($uids))
            $uids = array();
        else {
            $translations = $this->imap->getMessagesList();
            foreach ($translations as $translation)
                $this->uid_translation[$translation["uidl"]] = $translation["msg_id"];
        }

        return $uids;
    }

    function _translate_uid($uid)
    {
        return $this->uid_translation[$uid];
    }

    // Get header from mailbox
    function get_message_header($uid, $peek_for_body=true)
    {
        return $this->imap->getRawHeaders($this->_translate_uid($uid));
    }

    // Get message from mailbox
    function get_message_body($uid)
    {
        return $this->imap->getBody($this->_translate_uid($uid));
    }

    function get_message($uid)
    {
        $header = $this->get_message_header($uid);
        if ($header === false)
            return false;

        $body = $this->get_message_body($uid);
        if ($body === false)
            return false;

        print_r($header);
        print_r($body);

        return $header.$body;
    }

    function append_message($msg)
    {
        $this->imap->appendMessage($msg);
    }

    function move_message($uid, $new_folder)
    {
        // TODO:
        // return imap_mail_move($this->imap, $uid, $new_folder, CP_UID);

        echo "move_message not implemented";
        exit (0);
    }

    function delete_message($uid)
    {
        return $this->imap->deleteMessages($this->_translate_uid($uid));
    }

    function undelete_message($uid)
    {
        // TODO:
        // return imap_undelete($this->imap, $uid, FT_UID);
        echo "undelete not implemented";
        exit (0);
    }

    function expunge()
    {
        return $this->imap->expunge();
    }
}
