<?php
/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2 as     *
 *   published by the Free Software Foundation.                            *
 *                                                                         *
 *  Caching Kolab driver developed by KONSEC GmbH, sponsored by WU Wien    *
 *  copyright            : (c) 2006-2007 by Thomas Jarosch                 *
 *  email                : opensource@intra2net.com                        *
 ***************************************************************************/

// Kolab XML <-> array hash converter
//
// Preparse XML objects for cache storage
//
// All you have to do is inherit this class
// and provide a load/save function
class Kolab_XML
{
    // XML parser
    var $xmldoc = null;
    var $root_name = null;

    // Parse XML. Returns root node on success
    function parse_xml(&$xmltext)
    {
        $this->xmldoc = domxml_open_mem($xmltext, DOMXML_LOAD_PARSING +
                                DOMXML_LOAD_COMPLETE_ATTRS + DOMXML_LOAD_SUBSTITUTE_ENTITIES +
                                DOMXML_LOAD_DONT_KEEP_BLANKS, $error);
        if (!empty($error) || $this->xmldoc == null) {
            $this->xmldoc = null;
            return false;
        }

        return $this->xmldoc->document_element();
    }

    // Creates new XML document if necessary
    // Returns root node
    function prepare_save()
    {
        if ($this->xmldoc != null) {
            $root = $this->xmldoc->document_element();
        } else {
            // create new XML
            $this->xmldoc = domxml_new_doc("1.0");
            $root = $this->xmldoc->create_element($this->root_name);
            $root = $this->xmldoc->append_child($root);
        }

        return $root;
    }

    function get_xml_data($childs, $search_name)
    {
        foreach ($childs as $child) {
            if($child->tagname == $search_name) {
                $value = $child->get_content();
                return String::convertCharset($value, "utf-8");
            }
        }

        return "";
    }

    function create_text_node($parent, $name, $value, $replace_node=false)
    {
        $value = String::convertCharset($value, NLS::getCharset(), "utf-8");

        $node = $this->xmldoc->create_element($name);

        if ($replace_node !== false)
            $replace_node->replace_node($node);
        else
            $node = $parent->append_child($node);

        // content
        $text = $this->xmldoc->create_text_node($value);
        $text = $node->append_child($text);

        return $node;
    }

    function find_node($nodes, $name)
    {
        foreach($nodes as $node)
            if ($node->tagname == $name)
                return $node;

        return false;
    }

    function find_node_by_child_data($nodes, $parent_name, $child_name, $value)
    {
        foreach($nodes as $node)
        {
            if ($node->tagname == $parent_name) {
                $childs = $node->child_nodes();
                foreach($childs as $child)
                    if ($child->tagname == $child_name && $child->get_content() == $value)
                        return $node;
            }
        }

        return false;
    }

    function update_node($parent_node, $childs, $attributes, $name, $allow_empty=false)
    {
        // this field is of no interest to us
        if (!array_key_exists($name, $attributes))
            return;

        $old_node = $this->find_node($childs, $name);
        $value = $attributes[$name];

        // remove existing node if current child is empty
        if (empty($value) && !$allow_empty) {
            if ($old_node !== false)
                $parent_node->remove_child($old_node);

            return;
        }

        // create or update node
        $this->create_text_node($parent_node, $name, $value, $old_node);
    }

    function create_parent($parent, $name)
    {
        $node = $this->xmldoc->create_element($name);
        $node = $parent->append_child($node);

        return $node;
    }

    function create_or_find_parent($parent, $childs, $name)
    {
        // look for existing node
        $old_node = $this->find_node($childs, $name);
        if ($old_node !== false)
            return $old_node;

        // create new parent node
        return $this->create_parent($parent, $name);
    }
}
