<?php
/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2 as     *
 *   published by the Free Software Foundation.                            *
 *                                                                         *
 *  Caching Kolab driver developed by KONSEC GmbH, sponsored by WU Wien    *
 *  copyright            : (c) 2006-2007 by Thomas Jarosch                 *
 *  email                : opensource@intra2net.com                        *
 ***************************************************************************/

require_once 'Horde/Kolab/XML.php';

// Converter class for Kolab contacts
class Kolab_XML_contact extends Kolab_XML
{
    // XML parser
    var $fields_name;
    var $fields_address;
    var $fields_basic;

    var $phone_types;
    var $address_types;

    // internal constants
    var $number_of_emails = 3;

    function Kolab_XML_contact()
    {
        $this->root_name = "contact";

        // all fields, in kolab format specification order
        //
        // A value of one indicates this field get's skipped during load
        // and has a special handler during save
        //
        $this->fields_basic = array();
        $this->fields_basic["uid"] = 0;
        $this->fields_basic["body"] = 0;
        $this->fields_basic["categories"] = 0;
        $this->fields_basic["creation-date"] = 0;
        $this->fields_basic["last-modification-date"] = 0;
        $this->fields_basic["sensitivity"] = 0;
        // $this->fields_basic["inline-attachment"] = 1;
        // $this->fields_basic["link-attachment"] = 1;
        $this->fields_basic["product-id"] = 0;

        $this->fields_basic["name"] = 1;

        $this->fields_basic["free-busy-url"] = 0;
        $this->fields_basic["organization"] = 0;
        $this->fields_basic["web-page"] = 0;
        $this->fields_basic["im-address"] = 0;
        $this->fields_basic["department"] = 0;

        $this->fields_basic["office-location"] = 0;
        $this->fields_basic["profession"] = 0;
        $this->fields_basic["job-title"] = 0;
        $this->fields_basic["manager-name"] = 0;

        $this->fields_basic["assistant"] = 0;
        $this->fields_basic["nick-name"] = 0;
        $this->fields_basic["spouse-name"] = 0;
        $this->fields_basic["birthday"] = 0;
        $this->fields_basic["anniversary"] = 0;
        $this->fields_basic["picture"] = 0;
        $this->fields_basic["children"] = 0;
        $this->fields_basic["gender"] = 0;
        $this->fields_basic["language"] = 0;
        $this->fields_basic["phone"] = 1;
        $this->fields_basic["email"] = 1;
        $this->fields_basic["address"] = 1;

        $this->fields_basic["preferred-address"] = 0;
        $this->fields_basic["latitude"] = 0;
        $this->fields_basic["longitude"] = 0;

        // name fields
        $this->fields_name = array();
        $this->fields_name[] = "given-name";
        $this->fields_name[] = "middle-name";
        $this->fields_name[] = "last-name";
        $this->fields_name[] = "full-name";
        $this->fields_name[] = "initials";
        $this->fields_name[] = "prefix";
        $this->fields_name[] = "suffix";

        // address fields
        $this->fields_address = array();
        $this->fields_address[] = "street";
        $this->fields_address[] = "locality";
        $this->fields_address[] = "region";
        $this->fields_address[] = "postal-code";
        $this->fields_address[] = "country";

        // address types
        $this->address_types = array();
        $this->address_types[] = "business";
        $this->address_types[] = "home";
        $this->address_types[] = "other";

        // phone types
        $this->phone_types = array();
        $this->phone_types[] = "business1";
        $this->phone_types[] = "business2";
        $this->phone_types[] = "businessfax";
        $this->phone_types[] = "callback";
        $this->phone_types[] = "car";
        $this->phone_types[] = "company";
        $this->phone_types[] = "home1";
        $this->phone_types[] = "home2";
        $this->phone_types[] = "homefax";
        $this->phone_types[] = "isdn";
        $this->phone_types[] = "mobile";
        $this->phone_types[] = "pager";
        $this->phone_types[] = "primary";
        $this->phone_types[] = "radio";
        $this->phone_types[] = "telex";
        $this->phone_types[] = "ttytdd";
        $this->phone_types[] = "assistant";
        $this->phone_types[] = "other";

    }

    function load(&$xmltext)
    {
        $noderoot = $this->parse_xml($xmltext);
        if ($noderoot === false)
            return false;

        $childs = $noderoot->child_nodes();

        // fresh contact
        $contact = array();
        $email_count = 1;

        // basic fields below the root node
        foreach($this->fields_basic as $simple_field => $special) {
            if ($special)
                continue;

            $data = $this->get_xml_data($childs, $simple_field);
            if (!empty($data))
                $contact[$simple_field] = $data;
        }

        // special fields
        foreach($childs as $child) {
            // name
            if ($child->tagname == "name") {
                $name_childs = $child->child_nodes();
                foreach($this->fields_name as $name_field) {
                    $data = $this->get_xml_data($name_childs, $name_field);
                    if (!empty($data))
                        $contact[$name_field] = $data;
                }
            // email
            } else if ($child->tagname == "email") {
                $email_childs = $child->child_nodes();
                // $display_name = $this->get_xml_data($email_childs, "display-name");
                $smtp_address = $this->get_xml_data($email_childs, "smtp-address");

                if (!empty($smtp_address) && $email_count <= $this->number_of_emails) {
                    $contact["email$email_count"] = $smtp_address;
                    $email_count++;
                }
            // phone
            } else if ($child->tagname == "phone") {
                $phone_childs = $child->child_nodes();

                // get type
                $type = $this->get_xml_data($phone_childs, "type");
                $number = $this->get_xml_data($phone_childs, "number");

                if (!empty($number) && array_search($type, $this->phone_types) !== false)
                    $contact["phone-".$type] = $number;
            // address
            } else if ($child->tagname == "address") {
                $address_childs = $child->child_nodes();

                // get type
                $type = $this->get_xml_data($address_childs, "type");

                // load fields
                $addr = array();
                foreach($this->fields_address as $name)
                    $addr[$name] = $this->get_xml_data($address_childs, $name);

                if (array_search($type, $this->address_types) !== false) {
                    foreach($addr as $name => $value)
                        if (!empty($value))
                            $contact["addr-".$type."-".$name] = $value;
                }
            }
        }

        // Compatibility with broken clients
        $broken_fields = array("website" => "web-page",
                               "im-adress" => "im-address");
        foreach ($broken_fields as $broken_field => $real_field) {
            $data = $this->get_xml_data($childs, $broken_field);
            if (!empty($data) && empty($contact[$real_field]))
                $contact[$real_field] = $data;
        }

        // uid is vital
        if (empty($contact["uid"]))
            return false;

        return $contact;
    }

    function save($attributes)
    {
        $root = $this->prepare_save();
        $nodes = $root->child_nodes();

        // fill in default fields for new contacts
        if ($this->find_node($nodes, "creation-date") === false)
            $attributes["creation-date"] = gmstrftime("%Y-%m-%dT%H:%M:%SZ");
        if ($this->find_node($nodes, "sensitivity") === false)
            $attributes["sensitivity"] = "public";

        $attributes["last-modification-date"] = gmstrftime("%Y-%m-%dT%H:%M:%SZ");

        // create XML nodes
        foreach($this->fields_basic as $basic_field => $special) {
            // basic fields below the root node
            if (!$special) {
                $this->update_node($root, $nodes, $attributes, $basic_field);
            } else if ($basic_field == "name") {
                $name_node = $this->create_or_find_parent($root, $nodes, $basic_field);
                $name_childs = $name_node->child_nodes();
                foreach($this->fields_name as $name_field)
                    $this->update_node($name_node, $name_childs, $attributes, $name_field);
            } else if ($basic_field == "email") {
                // find all email nodes
                $email_nodes = array();
                foreach($nodes as $node)
                    if ($node->tagname == $basic_field)
                        $email_nodes[] = $node;

                // update email nodes
                $current_email_node = 0;
                $email_node_count = count($email_nodes);
                $email_node_count > 0 ? $update_nodes = true : $update_nodes = false;

                for ($i = 1; $i <= $this->number_of_emails; $i++) {
                    $field = "email$i";
                    $address = $attributes[$field];

                    // delete email node if address is empty
                   if (empty($address) && $update_nodes) {
                        $root->remove_child($email_nodes[$current_email_node]);
                    } else if (!empty($address)) {
                        if ($update_nodes)
                            $node_email = $email_nodes[$current_email_node];
                        else
                            $node_email = $this->create_parent($root, $basic_field);

                        $email_childs = $node_email->child_nodes();
                        $new_data = array("display-name" => $attributes["full-name"],
                                          "smtp-address" => $address);
                        $this->update_node($node_email, $email_childs, $new_data, "display-name");
                        $this->update_node($node_email, $email_childs, $new_data, "smtp-address");
                    }

                    if ($current_email_node < $email_node_count-1)
                        $current_email_node++;
                    else
                        $update_nodes = false;
                }
            } else if ($basic_field == "phone") {
                foreach($this->phone_types as $key => $type) {
                    $phone_name = "phone-$type";
                    if (array_key_exists($phone_name, $attributes)) {
                        $number = $attributes[$phone_name];

                        $node_phone = $this->find_node_by_child_data($nodes, $basic_field, "type", $type);
                        if ($node_phone === false) {
                            // skip this entry if the number is empty
                            if (empty($number))
                                continue;

                            $node_phone = $this->create_parent($root, $basic_field);
                        }

                        if (empty($number)) {
                            $root->remove_child($node_phone);
                        } else {
                            $phone_childs = $node_phone->child_nodes();
                            $new_data = array("type" => $type, "number" => $number);
                            $this->update_node($node_phone, $phone_childs, $new_data, "type");
                            $this->update_node($node_phone, $phone_childs, $new_data, "number");
                        }
                    }
                }
            } else if ($basic_field == "address") {
                foreach($this->address_types as $type) {
                    $new_addr = array();
                    $all_empty = true;

                    // build new address array
                    foreach($this->fields_address as $addr_field) {
                        $name = "addr-".$type."-".$addr_field;
                        $value = $attributes[$name];
                        $new_addr[$addr_field] = $value;

                        if (!empty($value))
                            $all_empty = false;
                    }

                    // search for existing address
                    $node_address = $this->find_node_by_child_data($nodes, $basic_field, "type", $type);
                    if ($node_address === false) {
                        if ($all_empty)
                            continue;

                        $node_address = $this->create_parent($root, $basic_field);
                    }

                    // delete or update address?
                    if ($all_empty) {
                        $root->remove_child($node_address);
                    } else {
                        $new_addr["type"] = $type;
                        $address_childs = $node_address->child_nodes();
                        foreach($new_addr as $addr_field => $dummy)
                            $this->update_node($node_address, $address_childs, $new_addr, $addr_field, true);
                    }
                }
            }
        }

        return $this->xmldoc->dump_mem(true);
    }
}
