<?php
/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2 as     *
 *   published by the Free Software Foundation.                            *
 *                                                                         *
 *  Caching Kolab driver developed by KONSEC GmbH, sponsored by WU Wien    *
 *  copyright            : (c) 2006-2007 by Thomas Jarosch                 *
 *  email                : opensource@intra2net.com                        *
 ***************************************************************************/

require_once 'Horde/Kolab/XML.php';
require_once 'Horde/Kolab/Util.php';

// Converter class for Kolab events
class Kolab_XML_event extends Kolab_XML
{
    // XML parser
    var $fields_name;
    var $fields_recurrence;
    var $fields_attendee;

    function Kolab_XML_event($imap, $imap_server)
    {
        $this->root_name = "event";

        // all (simple) fields, in kolab format specification order
        //
        // A value of one indicates this field get's skipped during load
        // and has a special handler during save
        //
        $this->fields_basic = array();
        $this->fields_basic["uid"] = 0;
        $this->fields_basic["body"] = 0;
        $this->fields_basic["categories"] = 0;
        $this->fields_basic["creation-date"] = 0;
        $this->fields_basic["last-modification-date"] = 0;
        $this->fields_basic["sensitivity"] = 0;
        // $this->fields_basic["inline-attachment"] = 1;
        // $this->fields_basic["link-attachment"] = 1;
        $this->fields_basic["product-id"] = 0;

        $this->fields_basic["summary"] = 0;
        $this->fields_basic["location"] = 0;
        $this->fields_basic["creator"] = 1;
        $this->fields_basic["organizer"] = 1;
        $this->fields_basic["start-date"] = 0;
        $this->fields_basic["alarm"] = 0;
        $this->fields_basic["recurrence"] = 1;
        $this->fields_basic["attendee"] = 1;
        $this->fields_basic["show-time-as"] = 0;
        $this->fields_basic["color-label"] = 0;
        $this->fields_basic["end-date"] = 0;

        // recurrence
        $this->fields_recurrence = array();
        // $this->fields_recurrence[] = "cycle";
        // $this->fields_recurrence[] = "type";
        $this->fields_recurrence[] = "interval";
        // $this->fields_recurrence[] = "day";
        $this->fields_recurrence[] = "daynumber";
        $this->fields_recurrence[] = "month";
        // $this->fields_recurrence[] = "range";
        // $this->fields_recurrence[] = "exclusion";

        // attendee
        $this->fields_attendee = array();
        $this->fields_attendee[] = "display-name";
        $this->fields_attendee[] = "smtp-address";
        $this->fields_attendee[] = "status";
        $this->fields_attendee[] = "request-response";
        $this->fields_attendee[] = "role";

        $this->xmldoc = null;
    }

    function load(&$xmltext)
    {
        $noderoot = $this->parse_xml($xmltext);
        if ($noderoot === false)
            return false;

        $childs = $noderoot->child_nodes();

        // fresh event
        $event = array();
        $email_count = 1;

        // basic fields below the root node
        foreach($this->fields_basic as $simple_field => $special) {
            if ($special)
                continue;

            $data = $this->get_xml_data($childs, $simple_field);
            if (!empty($data))
                $event[$simple_field] = $data;
        }

        // special fields
        foreach($childs as $child) {
            if ($child->tagname == "creator" || $child->tagname == "organizer") {
                $sub_childs = $child->child_nodes();
                $data = $this->get_xml_data($sub_childs, "smtp-address");
                if (!empty($data))
                    $event[$child->tagname] = $data;
            }
            // recurrence
            else if ($child->tagname == "recurrence") {
                $recurrence_childs = $child->child_nodes();
                $new_recurrence = array();

                // type
                $new_recurrence["cycle"] = $child->get_attribute("cycle");
                $new_recurrence["type"] = $child->get_attribute("type");

                // simple fields
                foreach($this->fields_recurrence as $recurrence_field) {
                    $data = $this->get_xml_data($recurrence_childs, $recurrence_field);
                    if (!empty($data))
                        $new_recurrence[$recurrence_field] = $data;
                }

                // special fields
                foreach($recurrence_childs as $child) {
                    if ($child->tagname == "day") {
                        $new_recurrence["days"][] = $child->get_content();
                    } else if ($child->tagname == "range") {
                        $range_type = $child->get_attribute("type");
                        $range = $child->get_content();

                        $new_recurrence["range-type"] = $range_type;
                        if ($range_type == "date")
                            $new_recurrence["range"] = Kolab_Util::decodeDate($range);
                        else
                            $new_recurrence["range"] = $range;
                    } else if ($child->tagname == "exclusion")
                        $new_recurrence["exclusions"][] = Kolab_Util::decodeDate($child->get_content());
                }

                $event["recurrence"] = $new_recurrence;
            }
            // attendee
            else if ($child->tagname == "attendee") {
                $attendee_childs = $child->child_nodes();
                $new_attendee = array();
                foreach($this->fields_attendee as $attendee_field) {
                    $data = $this->get_xml_data($attendee_childs, $attendee_field);
                    if (!empty($data))
                        $new_attendee[$attendee_field] = $data;
                }
                $event["attendees"][] = $new_attendee;
            }
        }

/*
        echo "<pre>Loaded:\n";
        print_r($event);
        echo "</pre>\n";
*/

        // convert dates
        $event["start-date"] = Kolab_Util::decodeDateOrDateTime($event["start-date"]);
        $event["end-date"] = Kolab_Util::decodeDateOrDateTime($event["end-date"]);

        // uid is vital
        if (empty($event["uid"]))
            return false;

        return $event;
    }

    function save($attributes)
    {
        $root = $this->prepare_save();
        $nodes = $root->child_nodes();

        // fill in default fields for new events
        if ($this->find_node($nodes, "creation-date") === false)
            $attributes["creation-date"] = gmstrftime("%Y-%m-%dT%H:%M:%SZ");
        if ($this->find_node($nodes, "sensitivity") === false)
            $attributes["sensitivity"] = "public";

        $attributes["last-modification-date"] = gmstrftime("%Y-%m-%dT%H:%M:%SZ");

        // create XML nodes
        foreach($this->fields_basic as $basic_field => $special) {
            // basic fields below the root node
            if (!$special) {
                $this->update_node($root, $nodes, $attributes, $basic_field);
            } else if ($basic_field == "creator") {
            } else if ($basic_field == "organizer") {
            } else if ($basic_field == "recurrence") {
                $recurrence_node = $this->find_node($nodes, $basic_field);
                if ($recurrence_node !== false)
                    $root->remove_child($recurrence_node);

                if (!isset($attributes["recurrence"]))
                    continue;

                $recurrence = $attributes["recurrence"];
                $recurrence_node = $this->create_parent($root, $basic_field);

                $recurrence_node->set_attribute("cycle", $recurrence["cycle"]);
                if (isset($recurrence["type"]))
                    $recurrence_node->set_attribute("type", $recurrence["type"]);

                $this->create_text_node($recurrence_node, "interval", $recurrence["interval"]);

                foreach($recurrence["days"] as $day)
                    $this->create_text_node($recurrence_node, "day", $day);

                if (isset($recurrence["daynumber"]))
                    $this->create_text_node($recurrence_node, "daynumber", $recurrence["daynumber"]);
                if (isset($recurrence["month"]))
                    $this->create_text_node($recurrence_node, "month", $recurrence["month"]);

                // range
                $range_node = $this->create_text_node($recurrence_node, "range", $recurrence["range"]);
                $range_node->set_attribute("type", $recurrence["range-type"]);

                foreach($recurrence["exclusions"] as $exclusion)
                    $this->create_text_node($recurrence_node, "exclusion", $exclusion);

                $this->fields_recurrence[] = "interval";
                // $this->fields_recurrence[] = "day";
                $this->fields_recurrence[] = "daynumber";
                $this->fields_recurrence[] = "month";
                // $this->fields_recurrence[] = "range";

            } else if ($basic_field == "attendee") {
                // update/delete attendies which are no in our list
                foreach ($nodes as $node) {
                    if ($node->tagname == $basic_field) {
                        $attendee_childs = $node->child_nodes();
                        $smtp_address = $this->get_xml_data($attendee_childs, "smtp-address");

                        $found_attendee = false;

                        foreach($attributes["attendees"] as $key => $attendee) {
                            if ($attendee["smtp-address"] == $smtp_address) {
                                foreach($this->fields_attendee as $attendee_field)
                                    $this->update_node($node, $attendee_childs, $attendee, $attendee_field);

                                $found_attendee = true;
                                unset($attributes["attendees"][$key]);
                                break;
                            }
                        }

                        if (!$found_attendee)
                            $root->remove_child($node);
                    }
                }

                // add new attendees
                foreach($attributes["attendees"] as $key => $attendee) {
                    $node_attendee = $this->create_parent($root, $basic_field);
                    $attendee_childs = $node_attendee->child_nodes();
                    foreach($this->fields_attendee as $attendee_field)
                        $this->update_node($node_attendee, $attendee_childs, $attendee, $attendee_field);
                }
            }
        }

        return $this->xmldoc->dump_mem(true);
    }
}
