<?php
/**
 * $Horde: turba/config/sources.php.dist,v 1.97.6.20 2006/06/20 09:38:33 jan Exp $
 *
 * This file is where you specify the sources of contacts available to users
 * at your installation. It contains a large number of EXAMPLES. Please
 * remove or comment out those examples that YOU DON'T NEED. There are a
 * number of properties that you can set for each server, including:
 *
 * title:       This is the common (user-visible) name that you want displayed
 *              in the contact source drop-down box.
 *
 * type:        The types 'ldap', 'sql', 'imsp' and 'prefs' are currently
 *              supported. Preferences-based address books are not intended
 *              for production installs unless you really know what you're
 *              doing - they are not searchable, and they won't scale well if
 *              a user has a large number of entries.
 *
 * params:      These are the connection parameters specific to the contact
 *              source. See below for examples of how to set these.
 *
 * Special params settings:
 *
 *   charset:       The character set that the backend stores data in. Many
 *                  LDAP servers use utf-8. Database servers typically use
 *                  iso-8859-1.
 *
 *   tls:           Only applies to LDAP servers. If true, then try to use a
 *                  TLS connection to the server.
 *
 *   scope:         Only applies to LDAP servers. Can be set to 'one' to
 *                  search one level of the LDAP directory, or 'sub' to search
 *                  all levels. 'one' will work for most setups and should be
 *                  much faster. However we default to 'sub' for backwards
 *                  compatibility.
 *
 *   checkrequired: Only applies to LDAP servers. If present, this value causes
 *                  the driver to consult the LDAP schema for any attributes
 *                  that are required by the given objectclass(es). Required
 *                  attributes will be provided automatically if the
 *                  'checkrequired_string' parameter is present.
 *
 *   checksyntax:   Only applies to LDAP servers. If present, this value causes
 *                  the driver to inspect the LDAP schema for particular
 *                  attributes by the type defined in the corresponding schema
 *
 *   filter:        Filter helps to filter your result based on certain
 *                  condition in SQL and LDAP backends. A filter can be
 *                  specified to avoid some unwanted data. For example, if the
 *                  source is an external sql database, to select records with
 *                  the delete flag = 0:
 *                  'filter' = 'deleted=0'
 *
 * map:         This is a list of mappings from the standard Turba attribute
 *              names (on the left) to the attribute names by which they are
 *              known in this contact source (on the right). Turba also
 *              supports composite fields. A composite field is defined by
 *              mapping the field name to an array containing a list of
 *              component fields and a format string (similar to a printf()
 *              format string). Here is an example:
 *              ...
 *              'name' => array('fields' => array('firstname', 'lastname'),
 *                              'format' => '%s %s'),
 *              'firstname' => 'object_firstname',
 *              'lastname' => 'object_lastname',
 *              ...
 *
 * tabs:        All fields can be grouped into tabs with this optional entry.
 *              This list is multidimensional hash, the keys are the tab
 *              titles.
 *              Here is an example:
 *              'tabs' => array(
 *                  'Names' => array('firstname', 'lastname', 'alias'),
 *                  'Addresses' => array('homeAddress', 'workAddress')
 *              );
 *
 * search:      A list of Turba attribute names that can be searched for this
 *              source.
 *
 * strict:      A list of native field/attribute names that must always be
 *              matched exactly in a search.
 *
 * export:      If set to true, this source will appear on the Export menu,
 *              allowing users to export the contacts to a CSV (etc.) file.
 *
 * browse:      If set to true, this source will be browseable via the Browse
 *              menu item, and empty searches against the source will return
 *              all contacts.
 *
 * use_shares:  If this is present and true, Horde_Share functionality will
 *              be enabled for this source - allowing users to share their
 *              personal address books as well as to create new ones. Since
 *              Turba only supports having one backend configured for
 *              creating new shares, use the 'shares' configuration option to
 *              specify which backend will be used for creating new shares.
 *              All permission checking will be done against Horde_Share, but
 *              note that any 'extended' permissions (such as max_contacts)
 *              will still be enforced. Also note that the backend driver
 *              must have support for using this. Currently SQL only.
 */

// Intranator IMAP based addressbook
$cfgSources['intranator'] = array(
     // replacements: %folder, %folder_without_namespace, %login, $server
     'title' => "Adressbuch \"%folder_without_namespace\"",
     'type' => 'intranator',
     'params' => array(
         'server' => '',
         'login' => '',
         'password' => '',
         'folder' => '',
         'cache_prefix' => '/tmp/imapcache/',
         // if the server is cyrus we can predict
         // the (shared) mailbox names and then
         // change our store filename so every
         // user benefits from cache updates
         'diskcache_cyrus_optimize' => true
     ),

     'map' => array(
        '__key' => 'uid',
        'name'              => 'full-name',
        'firstname'         => 'given-name',
        'lastname'          => 'last-name',
        'title'             => 'job-title',
        'company'           => 'organization',
        'notes'             => 'body',
        'website'           => 'web-page',
        'nickname'          => 'nick-name',
        'email1'            => 'email1',
        'email2'            => 'email2',
        'email3'            => 'email3',
        'homePhone'         => 'phone-home1',
        'workPhone'         => 'phone-business1',
        'cellPhone'         => 'phone-mobile',
        'fax'               => 'phone-businessfax',
        'workStreet'        => 'addr-business-street',
        'workCity'          => 'addr-business-locality',
        'workProvince'      => 'addr-business-region',
        'workPostalCode'    => 'addr-business-postal-code',
        'workCountry'       => 'addr-business-country',
        'homeStreet'        => 'addr-home-street',
        'homeCity'          => 'addr-home-locality',
        'homeProvince'      => 'addr-home-region',
        'homePostalCode'    => 'addr-home-postal-code',
        'homeCountry'       => 'addr-home-country',
     ),
     'search' => array(
            'name',
            'firstname',
            'lastname',
            'email1',
            'email2',
            'email3',
            'title',
            'company',
            'notes',
            'homePhone',
            'workPhone',
            'cellPhone',
            'fax',
            'workStreet',
            'workCity',
            'workProvince',
            'workPostalCode',
            'workCountry',
            'homeStreet',
            'homeCity',
            'homeProvince',
            'homePostalCode',
            'homeCountry',
            'website',
            'nickname'
     ),
     'strict' => array(
         'uid',
     ),
     'export' => true,
     'browse' => true,
 );

class kolab_contact_finder
{
    var $imap;
    var $template;
    var $server;
    var $login;
    var $password;
    var $default_folder;

    function check_folder_type($folder)
    {
        // TODO: Implement code for PHP versions without annotation patch
        //       Also we could search for folder type email in case
        //       annotations are supported but not set on this folder
        $annotation = imap_getannotation($this->imap, $folder, "/vendor/kolab/folder-type", "value.shared");

        $pos = strpos($annotation["value.shared"], "contact");
        if ($pos !== false && $pos == 0)
            return true;
        else
            return false;
    }

    function mbox_string()
    {
        return "{".$this->server.":143/notls}";
    }

    function create_contact_folder($folder)
    {
        $mbox = $this->mbox_string();

        // create folder
        if (!imap_createmailbox($this->imap, $mbox.$folder))
            return false;

        // set annotation
        if (!imap_setannotation($this->imap, $folder, "/vendor/kolab/folder-type", "value.shared", "contact"))
            return false;

        // store configuration email
        $msg  = "Subject: IMAP Folder configuration\r\nMIME-Version: 1.0\r\nX-Folderclass: IPF.Contact\r\n\r\n";
        $msg .= "Plase do not delete this message, it is used by Intra2net Connector\r\n";

        if (!imap_append($this->imap, $mbox.$folder, $msg))
            return false;

        return true;
    }

    function insert_config(&$cfgSources, &$kolab_sources, $folder)
    {
        $count = count($cfgSources) + 1;
        $entry = $cfgSources[$this->template];

        $charset_folder = String::convertCharset(imap_utf7_decode($folder), "iso-8859-1");
        $folder_without_namespace = $charset_folder;

        $pos_INBOX = strpos($folder_without_namespace, "INBOX/");
        $pos_user = strpos($folder_without_namespace, "user/");
        if ($pos_INBOX !== false && $pos_INBOX == 0)
            $folder_without_namespace = substr($folder_without_namespace, strlen("INBOX/"));
        else if ($pos_user !== false && $pos_user == 0)
            $folder_without_namespace = substr($folder_without_namespace, strlen("user/"));

        $entry["title"] = str_replace("%folder_without_namespace", $folder_without_namespace, $entry["title"]);
        $entry["title"] = str_replace("%folder", $charset_folder, $entry["title"]);

        $entry["title"] = str_replace("%login", $this->login, $entry["title"]);
        $entry["title"] = str_replace("%server", $this->server, $entry["title"]);
        $entry["params"]["server"] = $this->server;
        $entry["params"]["login"] = $this->login;
        $entry["params"]["password"] = $this->password;
        $entry["params"]["folder"] = $folder;

        $kolab_sources[$this->template.$count] = $entry;
        $cfgSources[$this->template.$count] = $entry;
    }

    function add_kolab_folders(&$cfgSources)
    {
        $mbox = $this->mbox_string();

        $this->imap = imap_open($mbox, $this->login, $this->password);
        $all_folders = imap_getmailboxes($this->imap, $mbox, "*");

        $add_folders = array();

        // look for contact folders
        $found_default_folder = false;
        foreach($all_folders as $folder) {
            if ($folder->attributes & LATT_NOSELECT)
                continue;

            $close_pos = strpos($folder->name, "}");
            if ($close_pos === false)
                continue;

            $short_folder = substr($folder->name, $close_pos+1);

            if ($this->check_folder_type($short_folder))
                $add_folders[] = $short_folder;

            if ($short_folder == $this->default_folder)
                $found_default_folder = true;
        }

        // create default contact folder if it's not there
        if (!$found_default_folder && !empty($this->default_folder) && $this->create_contact_folder($this->default_folder))
            $add_folders[] = $this->default_folder;

        // sort folder list so newly created folders appear at the right place
        sort ($add_folders);

        $kolab_sources = array();
        foreach ($add_folders as $folder)
            $this->insert_config($cfgSources, $kolab_sources, $folder);

        return $kolab_sources;
    }

    function kolab_contact_finder(&$cfgSources, $template, $server, $login, $password, $default_folder="")
    {
        $kolab_sources = "kolab_contact_finder_".$server."_".$login."_cfgsources";

        // Uncomment this for debugging
        unset($_SESSION[$kolab_sources]);

        if (isset($_SESSION[$kolab_sources])) {
            $cfgSources = array_merge($cfgSources, $_SESSION[$kolab_sources]);
            unset($cfgSources[$template]);
            return;
        }

        $this->template = $template;
        $this->server = $server;
        $this->login = $login;
        $this->password = $password;
        $this->default_folder = imap_utf7_encode($default_folder);

        if (!isset($cfgSources[$template]))
            echo "ERROR: Kolab contact template $template not found<br>\n";

        $_SESSION[$kolab_sources] = $this->add_kolab_folders($cfgSources);
        unset($cfgSources[$template]);
    }
}

$contacts_folder = $_SESSION["KOLAB_DEFAULT_CONTACTS_FOLDER"];
$finder = new kolab_contact_finder($cfgSources, "intranator", "127.0.0.1",
                                   sess_get_intranator_user(),
                                   sess_get_intranator_password(),
                                   $contacts_folder);
